#!/usr/bin/bash

CONF_FILE=/etc/squid/squid.conf
if [ ! -f $CONF_FILE ]; then
    # silently exit if no configuration file
    exit 0
fi
CACHE_LOG="`awk '$1 == "cache_log" {x=$2} END{print x}' $CONF_FILE`"
if [ -z "$CACHE_LOG" ]; then
    # silently exit if no cache_log setting in the configuration file
    # squid won't start without it
    exit 0
fi
CACHE_DIR="`dirname $CACHE_LOG`"
case "`basename $CACHE_DIR`" in
    *\$*) CACHE_DIR="`dirname $CACHE_DIR`" ;; # remove macro sub-part
esac
LOGFILE="$CACHE_DIR/squidcron.log"

# Read LARGE_ACCESS_LOG and SQUID_ variable settings from one of these files.
# Note that they need to be preceded by "export" to be usable.
if [ -f /etc/sysconfig/frontier-squid ]; then
    eval `egrep "^[^#]*(LARGE_ACCESS_LOG|SQUID_)" /etc/sysconfig/frontier-squid`
elif [ -f /etc/sysconfig/frontier-squid.sh ]; then
    eval `egrep "^[^#]*(LARGE_ACCESS_LOG|SQUID_)" /etc/sysconfig/frontier-squid.sh`
fi
if [ "$SQUID_MAX_ACCESS_LOG" = 0 ]; then
    # silently exit if SQUID_MAX_ACCESS_LOG is zero
    exit 0
fi

echo "Started rotateiflarge at `date`" >> $LOGFILE
/usr/sbin/fn-local-squid.sh rotateiflarge >> $LOGFILE 2>&1
echo "Finished rotateiflarge at `date`" >> $LOGFILE
